@@ -354,7 +354,7 @@ html_event_mouse_move (HtmlView *view, GdkEventMotion *event)
 		
 		/* And unref it */
 		if (cursor)
-			gdk_cursor_destroy (cursor);
+			gdk_cursor_unref (cursor);
 	}
 }
 
